@@ -33,7 +33,6 @@ using namespace Analitza;
 char Container::m_typeStr[][20] = {
 		"none",
 		"math",
-		"apply",
 		"declare",
 		"lambda",
 		"bvar",
@@ -48,7 +47,6 @@ char Container::m_typeStr[][20] = {
 QMap<QString, Container::ContainerType> createNameToType()
 {
 	QMap<QString, Container::ContainerType> ret;
-	ret["apply"]=Container::apply;
 	ret["declare"]=Container::declare;
 	ret["math"]=Container::math;
 	ret["lambda"]=Container::lambda;
@@ -69,10 +67,8 @@ Container::Container(const Container& c) : Object(Object::container), m_cont_typ
 {
 	Q_ASSERT(c.type()==Object::container);
 	
-	Container::const_iterator it=c.m_params.constBegin();
-	for(; it!=c.m_params.constEnd(); ++it) {
-		appendBranch((*it)->copy());
-	}
+	foreach(const Object* o, c.m_params)
+		appendBranch(o->copy());
 }
 
 Container* Container::copy() const
@@ -80,28 +76,11 @@ Container* Container::copy() const
 	return new Container(*this);
 }
 
-Operator Container::firstOperator() const
-{
-	foreach(const Object* o, m_params) {
-		if(o->type()==Object::oper)
-			return *static_cast<const Operator*>(o);
-	}
-	
-	return Operator(Operator::function);
-}
-
 QString Container::visit(ExpressionWriter* e) const
 {
 	return e->accept(this);
 }
 
-void Container::negate()
-{
-	foreach(Object* o, m_params) {
-		o->negate();
-	}
-}
-
 bool Container::isZero() const
 {
 	bool a=true;
@@ -119,19 +98,34 @@ Container::ContainerType Container::toContainerType(const QString& tag)
 QList<Ci*> Container::bvarCi() const
 {
 	QList<Ci*> ret;
-	QList<Object*>::const_iterator it, itEnd=firstValue();
+	QList<Object*>::const_iterator it, itEnd=m_params.constEnd();
 	
 	for(it=m_params.constBegin(); it!=itEnd; ++it) {
 		if((*it)->isContainer()) {
 			Container* c = (Container*) (*it);
-			if(c->containerType() == Container::bvar && !c->m_params.isEmpty() && c->m_params[0]->type()==Object::variable)
+			if(c->containerType() == Container::bvar) {
+				Q_ASSERT(!c->isEmpty() && c->m_params[0]->type()==Object::variable);
 				ret.append((Ci*) c->m_params[0]);
+			}
 		}
 	}
 	
 	return ret;
 }
 
+int Container::bvarCount() const
+{
+	int r=0;
+	QList<Object*>::const_iterator it, itEnd=m_params.constEnd();
+	for(it=m_params.constBegin(); it!=itEnd; ++it) {
+		Object* o = *it;
+		if(o->isContainer() && static_cast<Container*>(o)->containerType() == Container::bvar)
+			r++;
+	}
+	
+	return r;
+}
+
 QStringList Container::bvarStrings() const
 {
 	QStringList bvars;
@@ -143,33 +137,6 @@ QStringList Container::bvarStrings() const
 	return bvars;
 }
 
-Object* Container::ulimit() const
-{
-	Container* c=extractType(uplimit);
-	if(c)
-		return c->m_params.first();
-	else
-		return 0;
-}
-
-Object* Container::dlimit() const
-{
-	Container* c=extractType(downlimit);
-	if(c)
-		return c->m_params.first();
-	else
-		return 0;
-}
-
-Object* Container::domain() const
-{
-	Container* c=extractType(domainofapplication);
-	if(c)
-		return c->m_params.first();
-	else
-		return 0;
-}
-
 bool Container::operator==(const Container& c) const
 {
 	bool eq=c.m_params.count()==m_params.count();
@@ -183,7 +150,7 @@ bool Container::operator==(const Container& c) const
 
 bool Container::isNumber() const
 {
-	return m_cont_type==apply || m_cont_type==math || m_cont_type==lambda || m_cont_type==declare ||
+	return m_cont_type==math || m_cont_type==lambda || m_cont_type==declare ||
 		m_cont_type==piecewise || m_cont_type==piece || m_cont_type==otherwise;
 }
 
@@ -195,6 +162,7 @@ bool isValue(Object* o)
 		case Object::variable:
 		case Object::vector:
 		case Object::list:
+		case Object::apply:
 			ret=true;
 			break;
 		case Object::container:
@@ -208,44 +176,6 @@ bool isValue(Object* o)
 	return ret;
 }
 
-Container::iterator Container::firstValue()
-{
-	QList<Object *>::iterator it(m_params.begin()), itEnd(m_params.end());
-	for(; it!=itEnd; ++it) {
-		if(isValue(*it))
-			break;
-	}
-	
-	return it;
-}
-
-Container::const_iterator Container::firstValue() const
-{
-	QList<Object *>::const_iterator it(m_params.constBegin()), itEnd(m_params.constEnd());
-	for(; it!=itEnd; ++it) {
-		if(isValue(*it))
-			break;
-	}
-	
-	return it;
-}
-
-bool Container::isUnary() const
-{
-	QList<Object*>::const_iterator it(firstValue());
-	return ++it==m_params.end();
-}
-
-bool Container::isCorrect() const
-{
-	bool ret=m_type==Object::container && m_cont_type!=none;
-	if(m_cont_type==Container::apply) {
-		Operator o=firstOperator();
-		ret &= o.nparams()<0 || countValues()==o.nparams();
-	}
-	return ret;
-}
-
 QString Container::tagName() const
 {
 	return QString(m_typeStr[m_cont_type]);
@@ -268,17 +198,6 @@ bool Container::matches(const Object* exp, QMap<QString, const Object*>* found)
 	return matching;
 }
 
-int Container::countValues() const
-{
-	Container::const_iterator it=firstValue(), itEnd=m_params.constEnd();
-	int count=0;
-	
-	for(; it!=itEnd; ++it)
-		count++;
-	
-	return count;
-}
-
 Container* Container::extractType(Container::ContainerType t) const
 {
 	for(Container::const_iterator it=m_params.begin(); it!=m_params.end(); ++it) {
@@ -293,46 +212,3 @@ void Container::appendBranch(Object* o)
 {
 	m_params.append(o);
 }
-
-///Rename to decorate
-bool Container::decorate(const ScopeInformation& scope)
-{
-	if(m_cont_type==bvar)
-		return false;
-	
-	Container::const_iterator it=m_params.constBegin(), itEnd=m_params.constEnd();
-	if((*it)->type()==Object::oper)
-		++it;
-	
-	ScopeInformation newScope(scope);
-	QList<Ci*> bvars=bvarCi();
-	if(m_cont_type==declare) {
-		Q_ASSERT((*it)->type()==variable);
-		bvars.append((Ci*) *it);
-	}
-	
-	foreach(Ci* var, bvars) {
-		Object** o=new Object*(0);
-		var->setValue(o, true);
-		newScope.insert(var->name(), o);
-		++it;
-	}
-	
-	bool ret=false;
-	//We need to use the parent scope for (up/down)limit
-	for(; it!=itEnd;) {
-		Container *c = (Container*) (*it);
-		
-		if(c->isContainer() && (c->containerType()==uplimit ||  c->containerType()==downlimit)) {
-			ret |= (*it)->decorate(scope);
-			++it;
-		} else
-			break;
-	}
-	
-	for(; it!=itEnd; ++it) {
-		ret |= (*it)->decorate(newScope);
-	}
-	
-	return ret;
-}
