@@ -47,10 +47,10 @@
 #include <KToggleAction>
 #include <KToolBar>
 #include <KIcon>
-#include <KNS/Engine>
 #include <KApplication>
 #include <KGlobal>
 
+#include <knewstuff3/downloaddialog.h>
 //Project includes
 #include "ui_fontsdlg.h"
 #include "timer.h"
@@ -126,7 +126,7 @@ bool KLettres::loadLayout(QDomDocument &layoutDocument)
 
 void KLettres::setupActions()
 {
-    QAction *m_newAction = actionCollection()->addAction("play_new");
+    KAction *m_newAction = actionCollection()->addAction("play_new");
     m_newAction->setText(i18n("New Sound"));
     m_newAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_N));
     m_newAction->setIcon(KIcon("document-new")); // ### better icon for this?
@@ -139,7 +139,7 @@ void KLettres::setupActions()
     m_stuffAction->setIcon(KIcon("get-hot-new-stuff"));
     connect(m_stuffAction, SIGNAL(triggered(bool)),  this, SLOT( slotDownloadNewStuff() ));
 
-    QAction *m_playAgainAction = actionCollection()->addAction("play_again");
+    KAction *m_playAgainAction = actionCollection()->addAction("play_again");
     m_playAgainAction->setText(i18n("Replay Sound"));
     m_playAgainAction->setShortcut(QKeySequence(Qt::Key_F5));
     m_playAgainAction->setIcon(KIcon("media-playback-start"));
@@ -148,14 +148,7 @@ void KLettres::setupActions()
     m_playAgainAction->setWhatsThis(i18n("You can replay the same sound again by clicking this button or using the File menu, Replay Sound."));
     KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
 
-    m_menubarAction = actionCollection()->add<KToggleAction>("menubar");
-    m_menubarAction->setText(i18n("Show &Menubar"));
-    m_menubarAction->setIcon(KIcon("edit-clear"));
-    m_menubarAction->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_M));
-    connect(m_menubarAction, SIGNAL(triggered(bool)), this, SLOT(slotMenubar()));
-    m_menubarAction->setCheckedState(KGuiItem(i18n("Hide &Menubar")));
-    m_menubarAction->setChecked(true);
-    m_menubarAction->setWhatsThis(i18n("You can show or hide the menubar as you wish by clicking this button."));
+    m_menubarAction = KStandardAction::showMenubar(this, SLOT(slotMenubar()), actionCollection());
 
     m_levelAction = actionCollection()->add<KSelectAction>("levels");
     m_levelAction->setText(i18nc("@label:listbox which difficulty level to use", "L&evel"));
@@ -166,10 +159,10 @@ void KLettres::setupActions()
     m_languageAction->setText(i18nc("@label:listbox", "&Language"));
     m_languageAction->setItems(m_languageNames);
 
-    m_levelsNames.append(i18nc("@item:inlistbox choose level 1", "Level 1" ));
-    m_levelsNames.append(i18nc("@item:inlistbox choose level 2",  "Level 2" ));
-    m_levelsNames.append(i18nc("@item:inlistbox choose level 3",  "Level 3" ));
-    m_levelsNames.append(i18nc("@item:inlistbox choose level 4",  "Level 4" ));
+    m_levelsNames.append(i18ncp("@item:inlistbox choose level", "Level %1" , "Level %1", 1));
+    m_levelsNames.append(i18ncp("@item:inlistbox choose level", "Level %1" , "Level %1", 2));
+    m_levelsNames.append(i18ncp("@item:inlistbox choose level", "Level %1" , "Level %1", 3));
+    m_levelsNames.append(i18ncp("@item:inlistbox choose level", "Level %1" , "Level %1", 4));
     m_levelAction->setItems(m_levelsNames);
     m_levelAction->setCurrentItem(Prefs::level()-1);
 
@@ -231,7 +224,7 @@ void KLettres::optionsPreferences()
     //fontsdlg is the page name, mFont is the widget name, Font Settings is the page display string
     //fonts is the icon
     Timer *m_timer = new Timer();
-    dialog->addPage(m_timer, i18n("Timer"), "clock");
+    dialog->addPage(m_timer, i18n("Timer"), "chronometer");
     connect(dialog, SIGNAL(settingsChanged( const QString &)), this, SLOT(slotUpdateSettings()));
     dialog->setAttribute( Qt::WA_DeleteOnClose );
     dialog->setHelp(QString(), "klettres");
@@ -266,9 +259,8 @@ void KLettres::loadSettings()
 
 void KLettres::slotDownloadNewStuff()
 {
-    KNS::Entry::List entries = KNS::Engine::download();
-    // we need to delete the entry* items in the returned list
-    qDeleteAll(entries);
+    KNS3::DownloadDialog dialog("klettres.knsrc", this);
+    dialog.exec();
 
     //look for languages dirs installed
     QStringList languages = LangUtils::getLanguages();
@@ -282,20 +274,7 @@ void KLettres::slotDownloadNewStuff()
 
 void KLettres::slotMenubar()
 {
-    switch (m_menubarAction->isChecked()){
-    case false:
-	m_menubarAction->setChecked(false);
-        m_menubarAction->setText(i18n("Show Menubar"));
-        m_menubarAction->setToolTip(i18n("Show Menubar"));
-        menuBar()->hide();
-        break;
-    case true:
-        m_menubarAction->setChecked(true);
-        m_menubarAction->setText(i18n("Hide Menubar"));
-        m_menubarAction->setToolTip(i18n("Hide Menubar"));
-        menuBar()->show();
-        break;
-    }
+    menuBar()->setVisible(m_menubarAction->isChecked());
     Prefs::setMenuBarBool(m_menubarAction->isChecked());
     Prefs::self()->writeConfig();
 }
@@ -368,8 +347,6 @@ void KLettres::slotModeGrownup()
     m_kidAction->setChecked(false);
     m_grownupAction->setToolTip(i18n("Grown-up mode is currently active"));
     m_kidAction->setToolTip(i18n("Switch to Kid mode"));
-    m_menubarAction->setText(i18n("Hide Menubar"));
-    m_menubarAction->setToolTip(i18n("Hide Menubar"));
     menuBar()->show();
     m_view->m_timer = Prefs::grownTimer();
     Prefs::setMode(Prefs::EnumMode::grownup);
@@ -389,8 +366,6 @@ void KLettres::slotModeKid()
     m_kidAction->setChecked(true);
     m_kidAction->setToolTip(i18n("Kid mode is currently active"));
     m_grownupAction->setToolTip(i18n("Switch to Grown-up mode"));
-    m_menubarAction->setText(i18n("Show Menubar"));
-    m_menubarAction->setToolTip(i18n("Show Menubar"));
     m_grownupAction->setChecked(false);
     menuBar()->hide();
     m_view->m_timer = Prefs::kidTimer();
