@@ -35,6 +35,7 @@
 #include <kpushbutton.h>
 #include <klineedit.h>
 #include <knuminput.h>
+#include <kactioncollection.h>
 
 #include "colorscheme.h"
 #include "kstars.h"
@@ -42,7 +43,7 @@
 #include "skymap.h"
 #include "skyobjects/skyobject.h"
 #include "skyobjects/ksplanetbase.h"
-#include "infoboxes.h"
+#include "skycomponents/skymapcomposite.h"
 #include "simclock.h"
 #include "Options.h"
 
@@ -59,39 +60,45 @@
 #endif
 
 void KStars::setRaDec( double ra, double dec ) {
-    map()->setDestination( new SkyPoint( ra, dec ) );
+    SkyPoint p( ra, dec );
+    map()->setDestination( &p );
 }
 
 void KStars::setAltAz( double alt, double az ) {
-    map()->setDestinationAltAz(alt,az);
+    map()->setDestinationAltAz( dms(alt), dms(az) );
 }
 
 void KStars::lookTowards ( const QString &direction ) {
     QString dir = direction.toLower();
-    if (dir == "zenith" || dir=="z") map()->invokeKey( Qt::Key_Z );
-    else if (dir == "north" || dir=="n") map()->invokeKey( Qt::Key_N );
-    else if (dir == "east"  || dir=="e") map()->invokeKey( Qt::Key_E );
-    else if (dir == "south" || dir=="s") map()->invokeKey( Qt::Key_S );
-    else if (dir == "west"  || dir=="w") map()->invokeKey( Qt::Key_W );
-    else if (dir == "northeast" || dir=="ne") {
+    if        (dir == "zenith" || dir=="z") {
+        actionCollection()->action("zenith")->trigger();
+    } else if (dir == "north"  || dir=="n") {
+        actionCollection()->action("north")->trigger();
+    } else if (dir == "east"   || dir=="e") {
+        actionCollection()->action("east")->trigger();
+    } else if (dir == "south"  || dir=="s") {
+        actionCollection()->action("south")->trigger();
+    } else if (dir == "west"   || dir=="w") {
+        actionCollection()->action("west")->trigger();
+    } else if (dir == "northeast" || dir=="ne") {
         map()->stopTracking();
         map()->clickedPoint()->setAlt( 15.0 ); map()->clickedPoint()->setAz( 45.0 );
-        map()->clickedPoint()->HorizontalToEquatorial( LST(), geo()->lat() );
+        map()->clickedPoint()->HorizontalToEquatorial( data()->lst(), data()->geo()->lat() );
         map()->slotCenter();
     } else if (dir == "southeast" || dir=="se") {
         map()->stopTracking();
         map()->clickedPoint()->setAlt( 15.0 ); map()->clickedPoint()->setAz( 135.0 );
-        map()->clickedPoint()->HorizontalToEquatorial( LST(), geo()->lat() );
+        map()->clickedPoint()->HorizontalToEquatorial( data()->lst(), data()->geo()->lat() );
         map()->slotCenter();
     } else if (dir == "southwest" || dir=="sw") {
         map()->stopTracking();
         map()->clickedPoint()->setAlt( 15.0 ); map()->clickedPoint()->setAz( 225.0 );
-        map()->clickedPoint()->HorizontalToEquatorial( LST(), geo()->lat() );
+        map()->clickedPoint()->HorizontalToEquatorial( data()->lst(), data()->geo()->lat() );
         map()->slotCenter();
     } else if (dir == "northwest" || dir=="nw") {
         map()->stopTracking();
         map()->clickedPoint()->setAlt( 15.0 ); map()->clickedPoint()->setAz( 315.0 );
-        map()->clickedPoint()->HorizontalToEquatorial( LST(), geo()->lat() );
+        map()->clickedPoint()->HorizontalToEquatorial( data()->lst(), data()->geo()->lat() );
         map()->slotCenter();
     } else {
         SkyObject *target = data()->objectNamed( direction );
@@ -107,7 +114,8 @@ void KStars::addLabel( const QString &name ) {
     SkyObject *target = data()->objectNamed( name );
     if ( target != NULL ) {
 		    data()->skyComposite()->addNameLabel( target );
-        if ( map()->transientObject() == target ) map()->setTransientObject( NULL );
+        if( map()->transientObject() == target )
+            map()->setTransientObject( NULL );
         map()->forceUpdate();
     }
 }
@@ -116,37 +124,46 @@ void KStars::removeLabel( const QString &name ) {
     SkyObject *target = data()->objectNamed( name );
     if ( target != NULL ) {
 		    data()->skyComposite()->removeNameLabel( target );
-        if ( map()->transientObject() == target ) map()->setTransientObject( NULL );
+        if( map()->transientObject() == target )
+            map()->setTransientObject( NULL );
         map()->forceUpdate();
     }
 }
 
 void KStars::addTrail( const QString &name ) {
-    SkyObject *target = data()->objectNamed( name );
-    if ( target != NULL && target->isSolarSystem() ) {
-        ((KSPlanetBase*)target)->addToTrail();
+    TrailObject *target = dynamic_cast<TrailObject*>( data()->objectNamed( name ) );
+    if( target ) {
+        target->addToTrail();
         map()->forceUpdate();
     }
 }
 
 void KStars::removeTrail( const QString &name ) {
-    SkyObject *target = data()->objectNamed( name );
-    if ( target != NULL && target->isSolarSystem() ) {
-        ((KSPlanetBase*)target)->clearTrail();
+    TrailObject *target = dynamic_cast<TrailObject*>( data()->objectNamed( name ) );
+    if ( target ) {
+        target->clearTrail();
         map()->forceUpdate();
     }
 }
 
 void KStars::zoom( double z ) {
-    if ( z > MAXZOOM ) z = MAXZOOM;
-    if ( z < MINZOOM ) z = MINZOOM;
-    Options::setZoomFactor( z );
-    reportZoom();
-    map()->forceUpdate();
+    map()->setZoomFactor( z );
+}
+
+void KStars::zoomIn() {
+    map()->slotZoomIn();
+}
+
+void KStars::zoomOut() {
+    map()->slotZoomOut();
+}
+
+void KStars::defaultZoom() {
+    map()->slotZoomDefault();
 }
 
 void KStars::setLocalTime(int yr, int mth, int day, int hr, int min, int sec) {
-    data()->changeDateTime( geo()->LTtoUT( KStarsDateTime( QDate(yr, mth, day), QTime(hr,min,sec) ) ) );
+    data()->changeDateTime( data()->geo()->LTtoUT( KStarsDateTime( QDate(yr, mth, day), QTime(hr,min,sec) ) ) );
 }
 
 void KStars::waitFor( double sec ) {
@@ -159,10 +176,10 @@ void KStars::waitForKey( const QString &k ) {
     data()->resumeKey = QKeySequence::fromString( k );
     if ( ! data()->resumeKey.isEmpty() ) {
         //When the resumeKey is pressed, resumeKey is set to empty
-        while ( ! data()->resumeKey.isEmpty() ) { qApp->processEvents(); }
-
+        while ( !data()->resumeKey.isEmpty() )
+            qApp->processEvents();
     } else {
-        kDebug() << i18n( "Error [DCOP waitForKey()]: Invalid key requested." );
+        kDebug() << i18n( "Error [D-Bus waitForKey()]: Invalid key requested." );
     }
 }
 
@@ -191,9 +208,6 @@ void KStars::setGeoLocation( const QString &city, const QString &province, const
 
             data()->setLocation( *loc );
 
-            //notify on-screen GeoBox
-            infoBoxes()->geoChanged( loc );
-
             //configure time zone rule
             KStarsDateTime ltime = loc->UTtoLT( data()->ut() );
             loc->tzrule()->reset_with_ltime( ltime, loc->TZ0(), data()->isTimeRunningForward() );
@@ -208,7 +222,7 @@ void KStars::setGeoLocation( const QString &city, const QString &province, const
             // If the sky is in Horizontal mode and not tracking, reset focus such that
             // Alt/Az remain constant.
             if ( ! Options::isTracking() && Options::useAltAz() ) {
-                map()->focus()->HorizontalToEquatorial( LST(), geo()->lat() );
+                map()->focus()->HorizontalToEquatorial( data()->lst(), data()->geo()->lat() );
             }
 
             // recalculate new times and objects
@@ -222,10 +236,10 @@ void KStars::setGeoLocation( const QString &city, const QString &province, const
 
     if ( !cityFound ) {
         if ( province.isEmpty() )
-            kDebug() << i18n( "Error [DCOP setGeoLocation]: city " ) << city << ", "
+            kDebug() << i18n( "Error [D-Bus setGeoLocation]: city " ) << city << ", "
             << country << i18n( " not found in database." ) << endl;
         else
-            kDebug() << i18n( "Error [DCOP setGeoLocation]: city " ) << city << ", "
+            kDebug() << i18n( "Error [D-Bus setGeoLocation]: city " ) << city << ", "
             << province << ", " << country << i18n( " not found in database." ) << endl;
     }
 }
@@ -238,7 +252,7 @@ void KStars::readConfig() {
 
     //Reset date, if one was stored
     if ( data()->StoredDate.isValid() ) {
-        data()->changeDateTime( geo()->LTtoUT( data()->StoredDate ) );
+        data()->changeDateTime( data()->geo()->LTtoUT( data()->StoredDate ) );
         data()->StoredDate = KDateTime(); //invalidate StoredDate
     }
 
@@ -249,14 +263,14 @@ void KStars::writeConfig() {
     Options::self()->writeConfig();
 
     //Store current simulation time
-    data()->StoredDate.setDJD( data()->lt().djd() );
+    data()->StoredDate = data()->lt();
 }
 
 QString KStars::getOption( const QString &name ) {
     //Some config items are not stored in the Options object while
     //the program is running; catch these here and returntheir current value.
-    if ( name == "FocusRA" ) { return QString::number( map()->focus()->ra()->Hours(), 'f', 6 ); }
-    if ( name == "FocusDec" ) { return QString::number( map()->focus()->dec()->Degrees(), 'f', 6 ); }
+    if ( name == "FocusRA" ) { return QString::number( map()->focus()->ra().Hours(), 'f', 6 ); }
+    if ( name == "FocusDec" ) { return QString::number( map()->focus()->dec().Degrees(), 'f', 6 ); }
 
     KConfigSkeletonItem *it = Options::self()->findItem( name );
     if ( it ) return it->property().toString();
@@ -287,15 +301,14 @@ void KStars::changeViewOption( const QString &op, const QString &val ) {
     if ( op == "ShadeTimeBox"    && bOk ) Options::setShadeTimeBox(    bVal );
     if ( op == "ShadeGeoBox"     && bOk ) Options::setShadeGeoBox(     bVal );
     if ( op == "ShadeFocusBox"   && bOk ) Options::setShadeFocusBox(   bVal );
-    if ( op == "ShowMainToolBar" && bOk ) Options::setShowMainToolBar( bVal );
-    if ( op == "ShowViewToolBar" && bOk ) Options::setShowViewToolBar( bVal );
 
     //[View]
-    if ( op == "FOVName"                ) Options::setFOVName(         val );
-    if ( op == "FOVSizeX"         && dOk ) Options::setFOVSizeX( (float)dVal );
-    if ( op == "FOVSizeY"         && dOk ) Options::setFOVSizeY( (float)dVal );
-    if ( op == "FOVShape"        && nOk ) Options::setFOVShape(       nVal );
-    if ( op == "FOVColor"               ) Options::setFOVColor(        val );
+    // FIXME: REGRESSION
+//     if ( op == "FOVName"                ) Options::setFOVName(         val );
+//     if ( op == "FOVSizeX"         && dOk ) Options::setFOVSizeX( (float)dVal );
+//     if ( op == "FOVSizeY"         && dOk ) Options::setFOVSizeY( (float)dVal );
+//     if ( op == "FOVShape"        && nOk ) Options::setFOVShape(       nVal );
+//     if ( op == "FOVColor"               ) Options::setFOVColor(        val );
     if ( op == "ShowStars"       && bOk ) Options::setShowStars(      bVal );
     if ( op == "ShowMessier"     && bOk ) Options::setShowMessier(    bVal );
     if ( op == "ShowMessierImages" && bOk ) Options::setShowMessierImages( bVal );
@@ -331,6 +344,7 @@ void KStars::changeViewOption( const QString &op, const QString &val ) {
     if ( op == "ShowPlanetNames"    && bOk ) Options::setShowPlanetNames(    bVal );
     if ( op == "ShowPlanetImages"   && bOk ) Options::setShowPlanetImages(   bVal );
     if ( op == "HideOnSlew"  && bOk ) Options::setHideOnSlew(  bVal );
+    if ( op == "ObsListSaveImage"  && bOk ) Options::setObsListSaveImage(  bVal );
     if ( op == "HideStars"   && bOk ) Options::setHideStars(   bVal );
     if ( op == "HidePlanets" && bOk ) Options::setHidePlanets( bVal );
     if ( op == "HideMessier" && bOk ) Options::setHideMessier( bVal );
@@ -408,6 +422,8 @@ void KStars::loadColorScheme( const QString &name ) {
             QApplication::setPalette( OriginalPalette );
         }
 
+        Options::setColorSchemeFile( name );
+
         map()->forceUpdate();
     }
 }
@@ -517,17 +533,17 @@ void KStars::printImage( bool usePrintDialog, bool useChartColors ) {
         //(if requested)
         ColorScheme cs;
         if ( useChartColors ) {
-            cs.copy( * data()->colorScheme() );
+            cs = *data()->colorScheme();
             loadColorScheme( "chart.colors" );
         }
 
-        map()->setMapGeometry();
+        map()->setupProjector();
         map()->exportSkyImage( &printer );
 
         //Restore old color scheme if necessary
         //(if printing was aborted, the colorscheme is still restored)
         if ( useChartColors ) {
-            data()->colorScheme()->copy( cs );
+            *(data()->colorScheme()) = cs;
             map()->forceUpdate();
         }
 
@@ -740,8 +756,7 @@ void KStars::setINDITargetName(const QString &objectName)
     INDI_P *prop;
     INDI_E *el;
 
-    if (!indidriver || !indimenu)
-    {
+    if (!indidriver || !indimenu) {
         kDebug() << "setINDITarget: establishINDI() failed.";
         return;
     }
@@ -752,8 +767,7 @@ void KStars::setINDITargetName(const QString &objectName)
     dev = indimenu->findDevice(indimenu->getCurrentDevice());
     if (!dev)
         dev = indimenu->findDeviceByLabel(indimenu->getCurrentDevice());
-    if (!dev)
-    {
+    if (!dev) {
         kDebug() << "Device " << indimenu->getCurrentDevice() << " not found!";
         return;
     }
@@ -761,20 +775,17 @@ void KStars::setINDITargetName(const QString &objectName)
     prop = dev->findProp("EQUATORIAL_EOD_COORD");
     if (!prop) return;
 
-    el   = prop->findElement("RA");
-    if (!el) return;
-    if (!el->write_w) return;
-
-    el->write_w->setText(QString::number(target->ra()->Hours()));
+    el = prop->findElement("RA");
+    if( !el || !el->write_w)
+        return;
+    el->write_w->setText(QString::number(target->ra().Hours()));
 
     el  = prop->findElement("DEC");
-    if (!el) return;
-    if (!el->write_w) return;
-
-    el->write_w->setText(QString::number(target->dec()->Degrees()));
+    if( !el || !el->write_w)
+        return;
+    el->write_w->setText(QString::number(target->dec().Degrees()));
 
     prop->newText();
-
 }
 
 
@@ -792,14 +803,14 @@ void KStars::setINDIAction(const QString &action)
     dev = indimenu->findDevice(indimenu->getCurrentDevice());
     if (!dev)
         dev = indimenu->findDeviceByLabel(indimenu->getCurrentDevice());
-    if (!dev)
-    {
+    if (!dev) {
         kDebug() << "Device " << indimenu->getCurrentDevice() << " not found!";
         return;
     }
 
     el = dev->findElem(action);
-    if (!el) return;
+    if( !el )
+        return;
 
     el->pp->activateSwitch(action);
 
