@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -17,13 +17,14 @@
 #include <cmath>
 
 // Qt
-#include <QtCore/QDebug>
 #include <QtGui/QImage>
 
 // Marble
+#include "GeoPainter.h"
 #include "MarbleDirs.h"
-#include "TextureTile.h"
-#include "TileLoader.h"
+#include "MarbleDebug.h"
+#include "StackedTileLoader.h"
+#include "TextureColorizer.h"
 #include "ViewParams.h"
 #include "ViewportParams.h"
 #include "AbstractProjection.h"
@@ -31,24 +32,51 @@
 
 using namespace Marble;
 
-MercatorScanlineTextureMapper::MercatorScanlineTextureMapper( TileLoader *tileLoader,
-                                                              QObject * parent )
+MercatorScanlineTextureMapper::MercatorScanlineTextureMapper( StackedTileLoader *tileLoader,
+                                                              QObject *parent )
     : AbstractScanlineTextureMapper( tileLoader, parent ),
+      m_repaintNeeded( true ),
       m_oldCenterLon( 0.0 ),
       m_oldYPaintedTop( 0 )
 {
+    connect( m_tileLoader, SIGNAL( tileUpdateAvailable( const TileId & ) ),
+             this, SIGNAL( tileUpdatesAvailable() ) );
+    connect( m_tileLoader, SIGNAL( tileUpdatesAvailable() ),
+             this, SIGNAL( tileUpdatesAvailable() ) );
 }
 
+void MercatorScanlineTextureMapper::mapTexture( GeoPainter *painter,
+                                                ViewParams *viewParams,
+                                                const QRect &dirtyRect,
+                                                TextureColorizer *texColorizer )
+{
+    if ( m_repaintNeeded ) {
+        mapTexture( viewParams, texColorizer );
 
-void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
+        m_repaintNeeded = false;
+    }
+
+    painter->drawImage( dirtyRect, *viewParams->canvasImage(), dirtyRect );
+}
+
+void MercatorScanlineTextureMapper::setRepaintNeeded()
+{
+    m_repaintNeeded = true;
+}
+
+void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams, TextureColorizer *texColorizer )
 {
     QImage       *canvasImage = viewParams->canvasImage();
+    const int imageHeight = canvasImage->height();
+    const int imageWidth  = canvasImage->width();
     const qint64  radius      = viewParams->radius();
 
-    const bool highQuality  = ( viewParams->mapQuality() == Marble::High
-                || viewParams->mapQuality() == Marble::Print );
-    const bool printQuality = ( viewParams->mapQuality() == Marble::Print );
+    const bool highQuality  = ( viewParams->mapQuality() == HighQuality
+                || viewParams->mapQuality() == PrintQuality );
+    const bool printQuality = ( viewParams->mapQuality() == PrintQuality );
 
+    //mDebug() << "m_maxGlobalX: " << m_maxGlobalX;
+    //mDebug() << "radius      : " << radius << endl;
     // Scanline based algorithm to do texture mapping
 
     // Initialize needed variables:
@@ -57,21 +85,20 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
 
     m_tilePosX = 65535;
     m_tilePosY = 65535;
-    m_toTileCoordinatesLon = (qreal)(globalWidth() / 2 - m_tilePosX);
-    m_toTileCoordinatesLat = (qreal)(globalHeight() / 2 - m_tilePosY);
 
     // Calculate how many degrees are being represented per pixel.
     const float rad2Pixel = (float)( 2 * radius ) / M_PI;
 
     // Reset backend
     m_tileLoader->resetTilehash();
-    selectTileLevel( viewParams );
+    setRadius( viewParams->radius() );
 
     // Evaluate the degree of interpolation
     const int n = interpolationStep( viewParams );
+    bool interpolate = false;
 
     bool interlaced = ( m_interlaced 
-            || viewParams->mapQuality() == Marble::Low );
+            || viewParams->mapQuality() == LowQuality );
 
     // Calculate translation of center point
     qreal centerLon, centerLat;
@@ -86,17 +113,17 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
 
     // Calculate y-range the represented by the center point, yTop and
     // what actually can be painted
-    yPaintedTop    = yTop = m_imageHeight / 2 - 2 * radius + yCenterOffset;
-    yPaintedBottom        = m_imageHeight / 2 + 2 * radius + yCenterOffset;
+    yPaintedTop    = yTop = imageHeight / 2 - 2 * radius + yCenterOffset;
+    yPaintedBottom        = imageHeight / 2 + 2 * radius + yCenterOffset;
  
     if (yPaintedTop < 0)                yPaintedTop = 0;
-    if (yPaintedTop > m_imageHeight)    yPaintedTop = m_imageHeight;
+    if (yPaintedTop > imageHeight)    yPaintedTop = imageHeight;
     if (yPaintedBottom < 0)             yPaintedBottom = 0;
-    if (yPaintedBottom > m_imageHeight) yPaintedBottom = m_imageHeight;
+    if (yPaintedBottom > imageHeight) yPaintedBottom = imageHeight;
 
     const qreal pixel2Rad = 1.0/rad2Pixel;
 
-    qreal leftLon = + centerLon - ( m_imageWidth / 2 * pixel2Rad );
+    qreal leftLon = + centerLon - ( imageWidth / 2 * pixel2Rad );
     while ( leftLon < -M_PI ) leftLon += 2 * M_PI;
     while ( leftLon >  M_PI ) leftLon -= 2 * M_PI;
 
@@ -124,7 +151,7 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
         int  xIpRight = n * (int)( xRight / n - 1 ) + 1; 
 
         lon = leftLon;
-        lat = atan( sinh( ( (m_imageHeight / 2 + yCenterOffset) - y )
+        lat = atan( sinh( ( (imageHeight / 2 + yCenterOffset) - y )
                     * pixel2Rad ) );
 
         for ( int x = xLeft; x < xRight; ++x ) {
@@ -133,23 +160,29 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
             if ( x >= xIpLeft && x <= xIpRight ) {
                 x += n - 1;
                 lon += (n - 1) * pixel2Rad;
-                m_interpolate = !printQuality;
+                interpolate = !printQuality;
             }
             else {
-                m_interpolate = false;
+                interpolate = false;
             }
 
             if ( lon < -M_PI ) lon += 2 * M_PI;
             if ( lon >  M_PI ) lon -= 2 * M_PI;
 
-            if ( m_interpolate ) {
-                pixelValueApprox( lon, lat, scanLine, n, highQuality );
+            if ( interpolate ) {
+                if (highQuality)
+                    pixelValueApproxF( lon, lat, scanLine, n );
+                else
+                    pixelValueApprox( lon, lat, scanLine, n );
 
                 scanLine += ( n - 1 );
             }
 
-            if ( x < m_imageWidth ) {
-                pixelValue( lon, lat, scanLine, highQuality );
+            if ( x < imageWidth ) {
+                if ( highQuality )
+                    pixelValueF( lon, lat, scanLine );
+                else
+                    pixelValue( lon, lat, scanLine );
             }
             m_prevLon = lon;
             m_prevLat = lat; // preparing for interpolation
@@ -161,7 +194,7 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
         // copy scanline to improve performance
         if ( interlaced && y + 1 < yPaintedBottom ) { 
 
-            int pixelByteSize = canvasImage->bytesPerLine() / m_imageWidth;
+            int pixelByteSize = canvasImage->bytesPerLine() / imageWidth;
 
             memcpy( canvasImage->scanLine( y + 1 ) + xLeft * pixelByteSize, 
                     canvasImage->scanLine( y ) + xLeft * pixelByteSize, 
@@ -172,7 +205,7 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
 
     // Remove unused lines
     const int clearStart = ( yPaintedTop - m_oldYPaintedTop <= 0 ) ? yPaintedBottom : 0;
-    const int clearStop  = ( yPaintedTop - m_oldYPaintedTop <= 0 ) ? m_imageHeight  : yTop;
+    const int clearStop  = ( yPaintedTop - m_oldYPaintedTop <= 0 ) ? imageHeight  : yTop;
 
     QRgb * const clearBegin = (QRgb*)( canvasImage->scanLine( clearStart ) );
     QRgb * const clearEnd = (QRgb*)( canvasImage->scanLine( clearStop ) );
@@ -186,6 +219,10 @@ void MercatorScanlineTextureMapper::mapTexture( ViewParams *viewParams )
     m_oldYPaintedTop = yPaintedTop;
 
     m_tileLoader->cleanupTilehash();
+
+    if ( texColorizer ) {
+        texColorizer->colorize( viewParams );
+    }
 }
 
 
