@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -76,8 +76,9 @@ public Q_SLOTS:
     void removeGeoDataDocument( int index );
 
  private:
-    QVector<GeoDataPlacemark> recurseContainer(GeoDataContainer container);
-    void createFilterProperties( QVector<GeoDataPlacemark> &container );
+    QVector<GeoDataPlacemark*> recurseContainer(GeoDataContainer *container);
+    void setupStyle( GeoDataDocument *doc, QVector<GeoDataPlacemark*> &container );
+    void createFilterProperties( QVector<GeoDataPlacemark*> &container );
     int cityPopIdx( qint64 population ) const;
     int spacePopIdx( qint64 diameter ) const;
     int areaPopIdx( qreal area ) const;
