@@ -26,18 +26,22 @@
 
 using namespace Marble;
 
+QAtomicInt VectorComposer::refCounter( 0 );
+PntMap *VectorComposer::s_coastLines( 0 );
+PntMap *VectorComposer::s_islands( 0 );
+PntMap *VectorComposer::s_lakeislands( 0 );
+PntMap *VectorComposer::s_lakes( 0 );
+PntMap *VectorComposer::s_glaciers( 0 );
+PntMap *VectorComposer::s_rivers( 0 );
+PntMap *VectorComposer::s_countries( 0 );
+PntMap *VectorComposer::s_usaStates( 0 );
+PntMap *VectorComposer::s_dateLine( 0 );
+bool VectorComposer::s_coastLinesLoaded( false );
+bool VectorComposer::s_overlaysLoaded( false );
+
 VectorComposer::VectorComposer( QObject * parent )
     : QObject( parent ),
       m_vectorMap( new VectorMap() ),
-      m_coastLines( new PntMap() ),
-      m_islands( new PntMap() ),
-      m_lakeislands( new PntMap() ),
-      m_lakes( new PntMap() ),
-      m_glaciers( new PntMap() ),
-      m_rivers( new PntMap() ),
-      m_countries( new PntMap() ),
-      m_usaStates( new PntMap() ),
-      m_dateLine( new PntMap() ),
       m_oceanPen( QPen( Qt::NoPen ) ),
       m_oceanBrush( QBrush( QColor( 153, 179, 204 ) ) ),
       m_landPen( QPen( Qt::NoPen ) ),
@@ -46,87 +50,117 @@ VectorComposer::VectorComposer( QObject * parent )
       m_textureLandBrush( QBrush( QColor( 255, 0, 0 ) ) ),
       m_textureGlacierBrush( QBrush( QColor( 0, 255, 0 ) ) ),
       m_textureLakeBrush( QBrush( QColor( 0, 0, 0 ) ) ),
-      m_dateLineBrush( QBrush( Qt::NoBrush ) ),
-      m_coastLinesLoaded( false ),
-      m_overlaysLoaded( false )
+      m_dateLineBrush( QBrush( Qt::NoBrush ) )
 {
+    if ( refCounter == 0 ) {
+        s_coastLinesLoaded = false;
+        s_overlaysLoaded = false;
+
+        s_coastLines = new PntMap();
+        s_islands = new PntMap();
+        s_lakeislands = new PntMap();
+        s_lakes = new PntMap();
+        s_glaciers = new PntMap();
+        s_rivers = new PntMap();
+        s_countries = new PntMap();
+        s_usaStates = new PntMap();
+        s_dateLine = new PntMap();
+    }
+    refCounter.ref();
+
     m_textureBorderPen.setStyle( Qt::SolidLine );
     m_textureBorderPen.setColor( QColor( 0, 255, 0 ) );
     m_dateLinePen.setStyle( Qt::DashLine );
     m_dateLinePen.setColor( QColor( 0, 0, 0 ) );
 
-    connect( m_coastLines, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_islands, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_lakeislands, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_lakes, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_glaciers, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_rivers, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_countries, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_usaStates, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
-    connect( m_dateLine, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_coastLines, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_islands, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_lakeislands, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_lakes, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_glaciers, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_rivers, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_countries, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_usaStates, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
+    connect( s_dateLine, SIGNAL( initialized() ), SIGNAL( datasetLoaded() ) );
 }
 
 VectorComposer::~VectorComposer()
 {
-    delete m_dateLine;
-    delete m_usaStates;    // The states of the USA
-    delete m_countries;    // The country borders
-    delete m_rivers;
-    delete m_glaciers;
-    delete m_lakes;
-    delete m_lakeislands;
-    delete m_islands;
-    delete m_coastLines;
     delete m_vectorMap;
+
+    refCounter.deref();
+    if (refCounter == 0) {
+        delete s_dateLine;
+        delete s_usaStates;    // The states of the USA
+        delete s_countries;    // The country borders
+        delete s_rivers;
+        delete s_glaciers;
+        delete s_lakes;
+        delete s_lakeislands;
+        delete s_islands;
+        delete s_coastLines;
+
+        s_coastLinesLoaded = false;
+        s_overlaysLoaded = false;
+    }
 }
 
 void VectorComposer::loadCoastlines()
 {
+    if ( s_coastLinesLoaded ) {
+        return;
+    }
+    s_coastLinesLoaded = true;
+
     mDebug() << Q_FUNC_INFO;
 
     // Coastlines
-    m_coastLines->load( MarbleDirs::path( "mwdbii/PCOAST.PNT" ) );
-    m_islands->load( MarbleDirs::path( "mwdbii/PISLAND.PNT" ) );
-    m_lakeislands->load( MarbleDirs::path( "mwdbii/PLAKEISLAND.PNT" ) );
-    m_lakes->load( MarbleDirs::path( "mwdbii/PLAKE.PNT" ) );
+    s_coastLines->load( MarbleDirs::path( "mwdbii/PCOAST.PNT" ) );
+    s_islands->load( MarbleDirs::path( "mwdbii/PISLAND.PNT" ) );
+    s_lakeislands->load( MarbleDirs::path( "mwdbii/PLAKEISLAND.PNT" ) );
+    s_lakes->load( MarbleDirs::path( "mwdbii/PLAKE.PNT" ) );
 
     // Ice and snow ...
-    m_glaciers->load( MarbleDirs::path( "mwdbii/PGLACIER.PNT" ) );
+    s_glaciers->load( MarbleDirs::path( "mwdbii/PGLACIER.PNT" ) );
 }
 
 void VectorComposer::loadOverlays()
 {
+    if ( s_overlaysLoaded ) {
+        return;
+    }
+    s_overlaysLoaded = true;
+
     mDebug() << Q_FUNC_INFO;
-    
+
     // The rivers.
-    m_rivers->load( MarbleDirs::path( "mwdbii/RIVER.PNT" ) );
+    s_rivers->load( MarbleDirs::path( "mwdbii/RIVER.PNT" ) );
 
     // The countries.
-    m_countries->load( MarbleDirs::path( "mwdbii/PDIFFBORDER.PNT" ) );
+    s_countries->load( MarbleDirs::path( "mwdbii/PDIFFBORDER.PNT" ) );
 
     // The States of the USA.
-    m_usaStates->load( MarbleDirs::path( "mwdbii/PUSA48.DIFF.PNT" ) );
+    s_usaStates->load( MarbleDirs::path( "mwdbii/PUSA48.DIFF.PNT" ) );
 
     // The date "line", which in reality is rather crooked.
-    m_dateLine->load( MarbleDirs::path( "mwdbii/DATELINE.PNT" ) );
+    s_dateLine->load( MarbleDirs::path( "mwdbii/DATELINE.PNT" ) );
 }
 
 void VectorComposer::drawTextureMap(ViewParams *viewParams)
 {
-    if ( !m_coastLinesLoaded ) {
-        m_coastLinesLoaded = true;
-        loadCoastlines();
-    }
-    
-    QPaintDevice  *origimg = viewParams->coastImage();
+    loadCoastlines();
+
+    QImage        *origimg = viewParams->coastImage();
     Quaternion     rotAxis = viewParams->planetAxis();
 
+    origimg->fill( Qt::transparent );
+
     //	m_vectorMap->clearNodeCount();
 
     bool antialiased = false;
 
-    if (   viewParams->mapQuality() == Marble::HighQuality
-        || viewParams->mapQuality() == Marble::PrintQuality )
+    if (   viewParams->mapQuality() == HighQuality
+        || viewParams->mapQuality() == PrintQuality )
     {
         antialiased = true;
     }
@@ -136,7 +170,7 @@ void VectorComposer::drawTextureMap(ViewParams *viewParams)
     m_vectorMap->setzPointLimit( 0 ); // 0.6 results in green pacific
 
     // Draw the coast line vectors
-    m_vectorMap->createFromPntMap( m_coastLines, viewParams->viewport() );
+    m_vectorMap->createFromPntMap( s_coastLines, viewParams->viewport() );
     m_vectorMap->setPen( m_textureLandPen );
     m_vectorMap->setBrush( m_textureLandBrush );
     m_vectorMap->drawMap( origimg, antialiased, viewParams->viewport(),
@@ -146,7 +180,7 @@ void VectorComposer::drawTextureMap(ViewParams *viewParams)
     m_vectorMap->setzBoundingBoxLimit( 0.8 );
     m_vectorMap->setzPointLimit( 0.9 );
 
-    m_vectorMap->createFromPntMap( m_islands, viewParams->viewport() );
+    m_vectorMap->createFromPntMap( s_islands, viewParams->viewport() );
     m_vectorMap->setPen( m_textureLandPen );
     m_vectorMap->setBrush( m_textureLandBrush );
     m_vectorMap->drawMap( origimg, antialiased, viewParams->viewport(),
@@ -161,13 +195,13 @@ void VectorComposer::drawTextureMap(ViewParams *viewParams)
          m_vectorMap->setzBoundingBoxLimit( 0.95 );
          m_vectorMap->setzPointLimit( 0.98 ); 
 
-         m_vectorMap->createFromPntMap( m_lakes, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_lakes, viewParams->viewport() );
          m_vectorMap->setPen( Qt::NoPen );
          m_vectorMap->setBrush( m_textureLakeBrush );
          m_vectorMap->drawMap( origimg, antialiased, viewParams->viewport(),
                                viewParams->mapQuality() );
 
-         m_vectorMap->createFromPntMap( m_lakeislands, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_lakeislands, viewParams->viewport() );
          m_vectorMap->setPen( Qt::NoPen );
          m_vectorMap->setBrush( m_textureLandBrush );
          m_vectorMap->drawMap( origimg, antialiased, viewParams->viewport(),
@@ -181,7 +215,7 @@ void VectorComposer::drawTextureMap(ViewParams *viewParams)
         // Glaciers
          m_vectorMap->setzBoundingBoxLimit( 0.8 );
          m_vectorMap->setzPointLimit( 0.9 );
-         m_vectorMap->createFromPntMap( m_glaciers, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_glaciers, viewParams->viewport() );
          m_vectorMap->setPen( Qt::NoPen );
          m_vectorMap->setBrush( m_textureGlacierBrush );
 
@@ -195,17 +229,14 @@ void VectorComposer::drawTextureMap(ViewParams *viewParams)
 void VectorComposer::paintBaseVectorMap( GeoPainter *painter, 
                                          ViewParams *viewParams )
 {
-    if ( !m_coastLinesLoaded ) {
-        m_coastLinesLoaded = true;
-        loadCoastlines();
-    }
+    loadCoastlines();
 
     Quaternion  rotAxis = viewParams->planetAxis();
 
     bool antialiased = false;
 
-    if (   viewParams->mapQuality() == Marble::HighQuality
-        || viewParams->mapQuality() == Marble::PrintQuality )
+    if (   viewParams->mapQuality() == HighQuality
+        || viewParams->mapQuality() == PrintQuality )
     {
 	antialiased = true;
     }
@@ -232,14 +263,14 @@ void VectorComposer::paintBaseVectorMap( GeoPainter *painter,
         m_vectorMap->setBrush( m_landBrush );
     }
 
-    m_vectorMap->createFromPntMap( m_coastLines, viewParams->viewport() );
+    m_vectorMap->createFromPntMap( s_coastLines, viewParams->viewport() );
     m_vectorMap->paintMap( painter, antialiased );
 
     // Islands
     m_vectorMap->setzBoundingBoxLimit( 0.8 );
     m_vectorMap->setzPointLimit( 0.9 );
 
-    m_vectorMap->createFromPntMap( m_islands, viewParams->viewport() );
+    m_vectorMap->createFromPntMap( s_islands, viewParams->viewport() );
 
     if ( showCoastlines ) {
         m_vectorMap->setPen( m_landPen );
@@ -262,12 +293,12 @@ void VectorComposer::paintBaseVectorMap( GeoPainter *painter,
          m_vectorMap->setzBoundingBoxLimit( 0.95 );
          m_vectorMap->setzPointLimit( 0.98 ); 
 
-         m_vectorMap->createFromPntMap( m_lakes, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_lakes, viewParams->viewport() );
          m_vectorMap->setPen( m_lakePen );
          m_vectorMap->setBrush( m_lakeBrush );
          m_vectorMap->paintMap( painter, antialiased );
 
-         m_vectorMap->createFromPntMap( m_lakeislands, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_lakeislands, viewParams->viewport() );
          m_vectorMap->setBrush( m_landBrush );
          m_vectorMap->paintMap( painter, antialiased );
     }
@@ -282,8 +313,8 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
 
     bool antialiased = false;
 
-    if (   viewParams->mapQuality() == Marble::HighQuality
-        || viewParams->mapQuality() == Marble::PrintQuality )
+    if (   viewParams->mapQuality() == HighQuality
+        || viewParams->mapQuality() == PrintQuality )
     {
         antialiased = true;
     }
@@ -294,15 +325,12 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
 
     if ( showCoastlines ) {
 
-        if ( !m_coastLinesLoaded ) {
-            m_coastLinesLoaded = true;
-            loadCoastlines();
-        }
+        loadCoastlines();
 
         m_vectorMap->setzBoundingBoxLimit( 0.4 );
         m_vectorMap->setzPointLimit( 0 ); // 0.6 results in green pacific
     
-        m_vectorMap->createFromPntMap( m_coastLines, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_coastLines, viewParams->viewport() );
         m_vectorMap->setPen( m_landPen );
         m_vectorMap->setBrush( Qt::NoBrush );
         m_vectorMap->paintMap( painter, antialiased );
@@ -310,7 +338,7 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
         m_vectorMap->setzBoundingBoxLimit( 0.8 );
         m_vectorMap->setzPointLimit( 0.9 );
 
-        m_vectorMap->createFromPntMap( m_islands, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_islands, viewParams->viewport() );
         m_vectorMap->setPen( m_landPen );
         m_vectorMap->setBrush( Qt::NoBrush );
         m_vectorMap->paintMap( painter, antialiased );
@@ -319,12 +347,12 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
         m_vectorMap->setzBoundingBoxLimit( 0.95 );
         m_vectorMap->setzPointLimit( 0.98 ); 
 
-        m_vectorMap->createFromPntMap( m_lakes, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_lakes, viewParams->viewport() );
         m_vectorMap->setPen( m_landPen );
         m_vectorMap->setBrush( Qt::NoBrush );
         m_vectorMap->paintMap( painter, antialiased );
 
-        m_vectorMap->createFromPntMap( m_lakeislands, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_lakeislands, viewParams->viewport() );
         m_vectorMap->paintMap( painter, antialiased );
     }
 
@@ -333,14 +361,11 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
     viewParams->propertyValue( "rivers", showRivers );
 
     if ( showWaterbodies && showRivers ) {
-        if ( !m_overlaysLoaded ) {
-            m_overlaysLoaded = true;
-            loadOverlays();
-        }
+        loadOverlays();
         // Rivers
          m_vectorMap->setzBoundingBoxLimit( -1.0 );
          m_vectorMap->setzPointLimit( -1.0 );
-         m_vectorMap->createFromPntMap( m_rivers, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_rivers, viewParams->viewport() );
 
          m_vectorMap->setPen( m_riverPen );
          m_vectorMap->setBrush( m_riverBrush );
@@ -351,14 +376,11 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
     viewParams->propertyValue( "borders", showBorders );
 
     if ( showBorders ) {
-        if ( !m_overlaysLoaded ) {
-            m_overlaysLoaded = true;
-            loadOverlays();
-        }
+        loadOverlays();
         // Countries
          m_vectorMap->setzBoundingBoxLimit( -1.0 );
          m_vectorMap->setzPointLimit( -1.0 );
-         m_vectorMap->createFromPntMap( m_countries, viewParams->viewport() );
+         m_vectorMap->createFromPntMap( s_countries, viewParams->viewport() );
 
         // Fancy Boundaries Hack:
         // FIXME: Find a clean solution that allows for all the 
@@ -377,8 +399,8 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
         QPen borderDashPen( Qt::black );
         m_vectorMap->setBrush( m_countryBrush );
 
-        if ( viewParams->mapQuality() == Marble::HighQuality 
-          || viewParams->mapQuality() == Marble::PrintQuality ) {
+        if ( viewParams->mapQuality() == HighQuality 
+          || viewParams->mapQuality() == PrintQuality ) {
 
             countryPen.setColor( penColor );
             m_vectorMap->setPen( countryPen );
@@ -394,9 +416,9 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
                 m_vectorMap->paintMap( painter, antialiased );
             }
         }
-        if ( viewParams->mapQuality() == Marble::OutlineQuality
-          || viewParams->mapQuality() == Marble::LowQuality
-          || viewParams->mapQuality() == Marble::NormalQuality ) {
+        if ( viewParams->mapQuality() == OutlineQuality
+          || viewParams->mapQuality() == LowQuality
+          || viewParams->mapQuality() == NormalQuality ) {
 
             if ( !showCoastlines ) {
                 countryPen.setWidthF( 1.0 );
@@ -409,11 +431,11 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
         // US-States
         m_vectorMap->setzBoundingBoxLimit( -1.0 );
         m_vectorMap->setzPointLimit( -1.0 );
-        m_vectorMap->createFromPntMap( m_usaStates, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_usaStates, viewParams->viewport() );
 
         QPen statePen( m_statePen);
-        if ( viewParams->mapQuality() == Marble::OutlineQuality
-          || viewParams->mapQuality() == Marble::LowQuality ) {
+        if ( viewParams->mapQuality() == OutlineQuality
+          || viewParams->mapQuality() == LowQuality ) {
             statePen.setStyle( Qt::SolidLine );
         }
         m_vectorMap->setPen( statePen );
@@ -423,11 +445,11 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
         // International Dateline
         m_vectorMap->setzBoundingBoxLimit( -1.0 );
         m_vectorMap->setzPointLimit( -1.0 );
-        m_vectorMap->createFromPntMap( m_dateLine, viewParams->viewport() );
+        m_vectorMap->createFromPntMap( s_dateLine, viewParams->viewport() );
 
         QPen dateLinePen( m_dateLinePen);
-        if ( viewParams->mapQuality() == Marble::OutlineQuality
-          || viewParams->mapQuality() == Marble::LowQuality ) {
+        if ( viewParams->mapQuality() == OutlineQuality
+          || viewParams->mapQuality() == LowQuality ) {
             dateLinePen.setStyle( Qt::SolidLine );
         }
         m_vectorMap->setPen( dateLinePen );
@@ -438,10 +460,4 @@ void VectorComposer::paintVectorMap( GeoPainter *painter,
     // mDebug() << "M_VectorMap calculated nodes: " << m_vectorMap->nodeCount();
 }
 
-
-void VectorComposer::resizeMap( int width, int height )
-{
-    m_vectorMap->resizeMap( width, height );
-}
-
 #include "VectorComposer.moc"
