@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -8,15 +8,17 @@
 // Copyright 2009      Patrick Spendrin <ps_ml@gmx.de>
 //
 
-#ifndef GEODATAPOLYGONPRIVATE_H
-#define GEODATAPOLYGONPRIVATE_H
+#ifndef MARBLE_GEODATAPOLYGONPRIVATE_H
+#define MARBLE_GEODATAPOLYGONPRIVATE_H
 
 #include "GeoDataGeometry_p.h"
 
+#include "GeoDataTypes.h"
+
 namespace Marble
 {
 
-class  GeoDataPolygonPrivate : public Marble::GeoDataGeometryPrivate
+class GeoDataPolygonPrivate : public GeoDataGeometryPrivate
 {
   public:
      GeoDataPolygonPrivate( TessellationFlags f )
@@ -57,4 +59,4 @@ class  GeoDataPolygonPrivate : public Marble::GeoDataGeometryPrivate
 
 } // namespace Marble
 
-#endif // GeoDataPolygonPRIVATE_H
+#endif
