@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -22,7 +22,7 @@ class GeoDataStyleMapPrivate
   public:
     QString nodeType() const
     {
-        return GeoDataTypes::GeoDataSyleMapType;
+        return GeoDataTypes::GeoDataStyleMapType;
     }
 
     QString lastKey;
