@@ -19,17 +19,23 @@
     Boston, MA 02110-1301, USA.
 */
 
-#ifndef DgmlVisibleTagHandler_h
-#define DgmlVisibleTagHandler_h
+#ifndef MARBLE_DGML_VISIBLETAGHANDLER_H
+#define MARBLE_DGML_VISIBLETAGHANDLER_H
 
 #include "GeoTagHandler.h"
 
-class DgmlVisibleTagHandler : public GeoTagHandler {
-public:
-    DgmlVisibleTagHandler();
-    virtual ~DgmlVisibleTagHandler();
+namespace Marble
+{
+namespace dgml
+{
 
+class DgmlVisibleTagHandler : public GeoTagHandler
+{
+public:
     virtual GeoNode* parse(GeoParser&) const;
 };
 
-#endif // DgmlVisibleTagHandler_h
+}
+}
+
+#endif
