@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -18,6 +18,7 @@
 
 #include "MarbleLocale.h"
 
+#include "global.h"
 
 namespace Marble
 {
@@ -27,9 +28,12 @@ class MarbleGlobalPrivate
 {
  public:
     MarbleGlobalPrivate();
+    
     virtual ~MarbleGlobalPrivate();
 
     MarbleLocale m_locale;
+    
+    MarbleGlobal::Profiles m_profiles;
 };
 
 }
