@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -11,23 +11,27 @@
 #include "RoutingWidget.h"
 
 #include "GeoDataLineString.h"
-#include "MarbleMap.h"
 #include "MarbleModel.h"
 #include "MarblePlacemarkModel.h"
 #include "MarbleWidget.h"
 #include "MarbleWidgetInputHandler.h"
-#include "RouteSkeleton.h"
+#include "RouteRequest.h"
 #include "RoutingInputWidget.h"
 #include "RoutingLayer.h"
 #include "RoutingManager.h"
 #include "RoutingModel.h"
+#include "RoutingProfilesModel.h"
+#include "RoutingProfileSettingsDialog.h"
 #include "RoutingProxyModel.h"
+#include "GeoDataDocument.h"
+#include "AlternativeRoutesModel.h"
 
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
-#include <QtGui/QFileDialog>
 #include <QtGui/QSortFilterProxyModel>
+#include <QtGui/QComboBox>
 #include <QtGui/QPainter>
+#include <QtGui/QFileDialog>
 
 #include "ui_RoutingWidget.h"
 
@@ -53,7 +57,7 @@ public:
 
     QSortFilterProxyModel *m_routingProxyModel;
 
-    RouteSkeleton *m_routeSkeleton;
+    RouteRequest *m_routeRequest;
 
     bool m_zoomRouteAfterDownload;
 
@@ -65,6 +69,8 @@ public:
 
     int m_currentFrame;
 
+    int m_iconSize;
+
     /** Constructor */
     RoutingWidgetPrivate();
 
@@ -91,11 +97,15 @@ private:
 RoutingWidgetPrivate::RoutingWidgetPrivate() :
         m_widget( 0 ), m_routingManager( 0 ), m_routingLayer( 0 ),
         m_activeInput( 0 ), m_inputRequest( 0 ), m_routingProxyModel( 0 ),
-        m_routeSkeleton( 0 ), m_zoomRouteAfterDownload( false ),
-        m_workOffline( false ), m_currentFrame( 0 )
+        m_routeRequest( 0 ), m_zoomRouteAfterDownload( false ),
+        m_workOffline( false ), m_currentFrame( 0 ),
+        m_iconSize( 16 )
 {
     createProgressAnimation();
     m_progressTimer.setInterval( 100 );
+    if ( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        m_iconSize = 32;
+    }
 }
 
 void RoutingWidgetPrivate::adjustInputWidgets()
@@ -113,17 +123,14 @@ void RoutingWidgetPrivate::adjustSearchButton()
     QString text = QObject::tr( "Get Directions" );
     QString tooltip = QObject::tr( "Retrieve routing instructions for the selected destinations." );
 
-    bool search = m_inputWidgets.size() < 2;
-    if ( m_inputWidgets.size() > 1 ) {
-        for ( int i = 0; i < m_inputWidgets.size(); ++i ) {
-            if ( !m_inputWidgets[i]->hasInput() ||
-                    ( m_inputWidgets[i]->hasInput() && !m_inputWidgets[i]->hasTargetPosition() ) ) {
-                search = true;
-            }
+    int validInputs = 0;
+    for ( int i = 0; i < m_inputWidgets.size(); ++i ) {
+        if ( m_inputWidgets[i]->hasTargetPosition() ) {
+            ++validInputs;
         }
     }
 
-    if ( search ) {
+    if ( validInputs < 2 ) {
         text = QObject::tr( "Search" );
         tooltip = QObject::tr( "Find places matching the search term" );
     }
@@ -146,14 +153,13 @@ void RoutingWidgetPrivate::setActiveInput( RoutingInputWidget *widget )
 void RoutingWidgetPrivate::createProgressAnimation()
 {
     // Size parameters
-    int const iconSize = 16;
-    qreal const h = iconSize / 2.0; // Half of the icon size
+    qreal const h = m_iconSize / 2.0; // Half of the icon size
     qreal const q = h / 2.0; // Quarter of the icon size
     qreal const d = 7.5; // Circle diameter
     qreal const r = d / 2.0; // Circle radius
 
     // Canvas parameters
-    QImage canvas( iconSize, iconSize, QImage::Format_ARGB32 );
+    QImage canvas( m_iconSize, m_iconSize, QImage::Format_ARGB32 );
     QPainter painter( &canvas );
     painter.setRenderHint( QPainter::Antialiasing, true );
     painter.setPen( QColor ( Qt::gray ) );
@@ -174,38 +180,47 @@ RoutingWidget::RoutingWidget( MarbleWidget *marbleWidget, QWidget *parent ) :
         QWidget( parent ), d( new RoutingWidgetPrivate )
 {
     d->m_ui.setupUi( this );
+    d->m_ui.routeComboBox->setVisible( false );
     d->m_widget = marbleWidget;
 
-    d->m_routeSkeleton = new RouteSkeleton( this );
-    d->m_routingManager = new RoutingManager( d->m_widget, this );
-    d->m_routingLayer = new RoutingLayer( d->m_widget, this );
-    d->m_routingLayer->setRouteSkeleton( d->m_routeSkeleton );
-    d->m_widget->map()->model()->addLayer( d->m_routingLayer );
+    d->m_routingManager = d->m_widget->model()->routingManager();
+    d->m_routeRequest = d->m_widget->model()->routingManager()->routeRequest();
+    d->m_ui.routeComboBox->setModel( d->m_routingManager->alternativeRoutesModel() );
 
-    connect( d->m_routingLayer, SIGNAL( routeDirty() ),
-             d->m_routingManager, SLOT( updateRoute() ) );
+    d->m_routingLayer = d->m_widget->routingLayer();
+    d->m_routingLayer->synchronizeAlternativeRoutesWith( d->m_ui.routeComboBox );
+
+    d->m_ui.routingProfileComboBox->setModel( d->m_routingManager->profilesModel() );
+
+    connect( d->m_routingManager->profilesModel(), SIGNAL( rowsInserted( QModelIndex, int, int ) ),
+             this, SLOT( selectFirstProfile() ) );
+    connect( d->m_routingManager->profilesModel(), SIGNAL( modelReset() ),
+             this, SLOT( selectFirstProfile() ) );
     connect( d->m_routingLayer, SIGNAL( placemarkSelected( QModelIndex ) ),
              this, SLOT( activatePlacemark( QModelIndex ) ) );
     connect( d->m_routingLayer, SIGNAL( pointSelected( GeoDataCoordinates ) ),
              this, SLOT( retrieveSelectedPoint( GeoDataCoordinates ) ) );
     connect( d->m_routingLayer, SIGNAL( pointSelectionAborted() ),
              this, SLOT( pointSelectionCanceled() ) );
-    connect( d->m_routingLayer, SIGNAL( exportRequested() ),
-             this, SLOT( exportRoute() ) );
-    connect( d->m_routingManager, SIGNAL( stateChanged( RoutingManager::State, RouteSkeleton* ) ),
-             this, SLOT( updateRouteState( RoutingManager::State, RouteSkeleton* ) ) );
-    connect( d->m_routeSkeleton, SIGNAL( positionAdded( int ) ),
+    connect( d->m_routingManager, SIGNAL( stateChanged( RoutingManager::State, RouteRequest* ) ),
+             this, SLOT( updateRouteState( RoutingManager::State, RouteRequest* ) ) );
+    connect( d->m_routeRequest, SIGNAL( positionAdded( int ) ),
              this, SLOT( insertInputWidget( int ) ) );
-    connect( d->m_routeSkeleton, SIGNAL( positionRemoved( int ) ),
+    connect( d->m_routeRequest, SIGNAL( positionRemoved( int ) ),
              this, SLOT( removeInputWidget( int ) ) );
     connect( &d->m_progressTimer, SIGNAL( timeout() ),
              this, SLOT( updateProgress() ) );
+    connect( d->m_ui.routeComboBox, SIGNAL( currentIndexChanged( int ) ),
+             this, SLOT( switchRoute( int ) ) );
+    connect( d->m_ui.routingProfileComboBox, SIGNAL( currentIndexChanged( int ) ),
+             this, SLOT( setRoutingProfile( int ) ) );
+    connect( d->m_routingManager->alternativeRoutesModel(), SIGNAL( rowsInserted( QModelIndex, int, int ) ),
+             this, SLOT( updateAlternativeRoutes() ) );
 
     d->m_routingProxyModel = new RoutingProxyModel( this );
     d->m_routingProxyModel->setSourceModel( d->m_routingManager->routingModel() );
     d->m_ui.directionsListView->setModel( d->m_routingProxyModel );
 
-    d->m_routingLayer->setModel( d->m_routingManager->routingModel() );
     QItemSelectionModel *selectionModel = d->m_ui.directionsListView->selectionModel();
     d->m_routingLayer->synchronizeWith( d->m_routingProxyModel, selectionModel );
     connect( d->m_ui.directionsListView, SIGNAL( activated ( QModelIndex ) ),
@@ -213,17 +228,23 @@ RoutingWidget::RoutingWidget( MarbleWidget *marbleWidget, QWidget *parent ) :
 
     connect( d->m_ui.searchButton, SIGNAL( clicked( ) ),
              this, SLOT( retrieveRoute () ) );
+    connect( d->m_ui.openButton, SIGNAL( clicked( bool ) ),
+             this, SLOT( openRouteFile() ) );
     connect( d->m_ui.optionsLabel, SIGNAL( linkActivated( QString ) ),
-             this, SLOT( toggleOptionsVisibility() ) );
+             this, SLOT( configureProfile() ) );
+    connect( d->m_ui.routeComboBox, SIGNAL( currentIndexChanged( int ) ),
+             this, SLOT( switchRoute( int ) ) );
 
-    addInputWidget();
-    addInputWidget(); // Start with source and destination
-    d->m_ui.routePreferenceComboBox->setVisible( false );
-    d->m_ui.highwaysCheckBox->setVisible( false );
-    d->m_ui.tollWaysCheckBox->setVisible( false );
-    d->m_ui.preferenceLabel->setVisible( false );
-    d->m_ui.avoidLabel->setVisible( false );
-    d->m_ui.descriptionLabel->setVisible( false );
+    for( int i=0; i<d->m_routeRequest->size(); ++i ) {
+        insertInputWidget( i );
+    }
+
+    for ( int i=0; i<2 && d->m_inputWidgets.size()<2; ++i ) {
+        // Start with source and destination if the route is empty yet
+        addInputWidget();
+    }
+    //d->m_ui.descriptionLabel->setVisible( false );
+    setOpenFileButtonVisible( false );
 }
 
 RoutingWidget::~RoutingWidget()
@@ -239,29 +260,13 @@ void RoutingWidget::retrieveRoute()
         return;
     }
 
-    int index = d->m_ui.routePreferenceComboBox->currentIndex();
-    RouteSkeleton::RoutePreference pref = RouteSkeleton::CarFastest;
-    if ( index == 1 ) {
-        pref = RouteSkeleton::CarShortest;
-    }
-    if ( index == 2 ) {
-        pref = RouteSkeleton::Bicycle;
-    }
-    if ( index == 3 ) {
-        pref = RouteSkeleton::Pedestrian;
-    }
-    RouteSkeleton::AvoidFeatures avoid = RouteSkeleton::AvoidNone;
-    if ( d->m_ui.highwaysCheckBox->isChecked() ) {
-        avoid |= RouteSkeleton::AvoidHighway;
-    }
-    if ( d->m_ui.tollWaysCheckBox->isChecked() ) {
-        avoid |= RouteSkeleton::AvoidTollWay;
+    int index = d->m_ui.routingProfileComboBox->currentIndex();
+    if ( index == -1 ) {
+        return;
     }
+    d->m_routeRequest->setRoutingProfile( d->m_routingManager->profilesModel()->profiles().at( index ) );
 
-    d->m_routeSkeleton->setRoutePreference( pref );
-    d->m_routeSkeleton->setAvoidFeatures( avoid );
-
-    Q_ASSERT( d->m_routeSkeleton->size() == d->m_inputWidgets.size() );
+    Q_ASSERT( d->m_routeRequest->size() == d->m_inputWidgets.size() );
     for ( int i = 0; i < d->m_inputWidgets.size(); ++i ) {
         RoutingInputWidget *widget = d->m_inputWidgets.at( i );
         if ( !widget->hasTargetPosition() && widget->hasInput() ) {
@@ -271,10 +276,10 @@ void RoutingWidget::retrieveRoute()
     }
 
     d->m_activeInput = 0;
-    if ( d->m_routeSkeleton->size() > 1 ) {
+    if ( d->m_routeRequest->size() > 1 ) {
         d->m_zoomRouteAfterDownload = true;
         d->m_routingLayer->setModel( d->m_routingManager->routingModel() );
-        d->m_routingManager->retrieveRoute( d->m_routeSkeleton );
+        d->m_routingManager->retrieveRoute( d->m_routeRequest );
         d->m_ui.directionsListView->setModel( d->m_routingProxyModel );
         d->m_routingLayer->synchronizeWith( d->m_routingProxyModel,
                                             d->m_ui.directionsListView->selectionModel() );
@@ -322,7 +327,7 @@ void RoutingWidget::handleSearchResult( RoutingInputWidget *widget )
 
     if ( placemarks.size() > 1 ) {
         d->m_widget->centerOn( GeoDataLatLonBox::fromLineString( placemarks ) );
-        d->m_ui.descriptionLabel->setVisible( false );
+        //d->m_ui.descriptionLabel->setVisible( false );
     }
 }
 
@@ -347,15 +352,13 @@ void RoutingWidget::activatePlacemark( const QModelIndex &index )
 
 void RoutingWidget::addInputWidget()
 {
-    int index = d->m_ui.routingLayout->count() - 4;
-    d->m_routeSkeleton->append( GeoDataCoordinates() );
-    insertInputWidget( index );
+    d->m_routeRequest->append( GeoDataCoordinates() );
 }
 
 void RoutingWidget::insertInputWidget( int index )
 {
     if ( index >= 0 && index <= d->m_inputWidgets.size() ) {
-        RoutingInputWidget *input = new RoutingInputWidget( d->m_routeSkeleton, index, this );
+        RoutingInputWidget *input = new RoutingInputWidget( d->m_widget, index, this );
         input->setProgressAnimation( d->m_progressAnimation );
         input->setWorkOffline( d->m_workOffline );
         d->m_inputWidgets.insert( index, input );
@@ -382,7 +385,7 @@ void RoutingWidget::removeInputWidget( RoutingInputWidget *widget )
         if ( d->m_inputWidgets.size() < 3 ) {
             widget->clear();
         } else {
-            d->m_routeSkeleton->remove( index );
+            d->m_routeRequest->remove( index );
         }
         d->m_routingManager->updateRoute();
     }
@@ -407,49 +410,17 @@ void RoutingWidget::removeInputWidget( int index )
     }
 }
 
-void RoutingWidget::updateRouteState( RoutingManager::State state, RouteSkeleton *route )
+void RoutingWidget::updateRouteState( RoutingManager::State state, RouteRequest *route )
 {
     Q_UNUSED( route );
 
-    if ( state == RoutingManager::Retrieved ) {
-        // Parts of the route may lie outside the route trip points
-        GeoDataLineString bbox;
-        for ( int i = 0; i < d->m_routingManager->routingModel()->rowCount(); ++i ) {
-            QModelIndex index = d->m_routingManager->routingModel()->index( i, 0 );
-            QVariant pos = index.data( RoutingModel::CoordinateRole );
-            if ( !pos.isNull() ) {
-                bbox << qVariantValue<GeoDataCoordinates>( pos );
-            }
-        }
-
-        if ( bbox.size() > 1 ) {
-            qreal distance = d->m_routingManager->routingModel()->totalDistance();
-            unsigned int days = d->m_routingManager->routingModel()->duration().days;
-            QTime time = d->m_routingManager->routingModel()->duration().time;
-            QString timeString = time.toString( Qt::DefaultLocaleShortDate );
-            if ( days ) {
-                QString label = tr( "Estimated travel time: %1 days, %2 (%3 km)", 0, days );
-                d->m_ui.descriptionLabel->setText( label.arg( days ).arg( timeString ).arg( distance, 0, 'f', 1 ) );
-            } else {
-                QString label = tr( "Estimated travel time: %1 (%2 km)" );
-                d->m_ui.descriptionLabel->setText( label.arg( timeString ).arg( distance, 0, 'f', 1 ) );
-            }
-            d->m_ui.descriptionLabel->setVisible( true );
-
-            if ( d->m_zoomRouteAfterDownload ) {
-                d->m_zoomRouteAfterDownload = false;
-                d->m_widget->centerOn( GeoDataLatLonBox::fromLineString( bbox ) );
-            }
-        }
+    if ( state != RoutingManager::Retrieved ) {
+        d->m_ui.routeComboBox->setVisible( false );
+        d->m_ui.routeComboBox->clear();
     }
 
-    d->m_routingLayer->setRouteDirty( state == RoutingManager::Downloading );
-
     if ( state == RoutingManager::Downloading ) {
         d->m_progressTimer.start();
-    } else {
-        d->m_progressTimer.stop();
-        d->m_ui.searchButton->setIcon( QIcon() );
     }
 }
 
@@ -489,28 +460,13 @@ void RoutingWidget::pointSelectionCanceled()
     }
 }
 
-void RoutingWidget::toggleOptionsVisibility()
+void RoutingWidget::configureProfile()
 {
-    bool visible = !d->m_ui.routePreferenceComboBox->isVisible();
-    d->m_ui.routePreferenceComboBox->setVisible( visible );
-    d->m_ui.highwaysCheckBox->setVisible( visible );
-    d->m_ui.tollWaysCheckBox->setVisible( visible );
-    d->m_ui.preferenceLabel->setVisible( visible );
-    d->m_ui.avoidLabel->setVisible( visible );
-}
-
-void RoutingWidget::exportRoute()
-{
-    QString fileName = QFileDialog::getSaveFileName( this, tr( "Export Route" ), // krazy:exclude=qclasses
-                       QDir::homePath(),
-                       tr( "GPX files (*.gpx)" ) );
-
-    if ( !fileName.isEmpty() ) {
-        QFile gpx( fileName );
-        if ( gpx.open( QFile::WriteOnly) ) {
-            d->m_routingManager->routingModel()->exportGpx( &gpx );
-            gpx.close();
-        }
+    int index = d->m_ui.routingProfileComboBox->currentIndex();
+    if ( index != -1 ) {
+        RoutingProfileSettingsDialog dialog( d->m_widget->model()->pluginManager(), d->m_routingManager->profilesModel(), d->m_widget );
+        dialog.editProfile( d->m_ui.routingProfileComboBox->currentIndex() );
+        d->m_routeRequest->setRoutingProfile( d->m_routingManager->profilesModel()->profiles().at( index ) );
     }
 }
 
@@ -533,6 +489,88 @@ void RoutingWidget::updateProgress()
     }
 }
 
+void RoutingWidget::switchRoute( int index )
+{
+    if ( index >= 0 )
+    {
+        Q_ASSERT( index < d->m_ui.routeComboBox->count() );
+        d->m_routingManager->alternativeRoutesModel()->setCurrentRoute( index );
+    }
+}
+
+void RoutingWidget::updateAlternativeRoutes()
+{
+    if ( d->m_ui.routeComboBox->count() == 1) {
+        // Parts of the route may lie outside the route trip points
+        GeoDataLineString bbox;
+        for ( int i = 0; i < d->m_routingManager->routingModel()->rowCount(); ++i ) {
+            QModelIndex index = d->m_routingManager->routingModel()->index( i, 0 );
+            QVariant pos = index.data( RoutingModel::CoordinateRole );
+            if ( !pos.isNull() ) {
+                bbox << qVariantValue<GeoDataCoordinates>( pos );
+            }
+        }
+
+        if ( bbox.size() > 1 ) {
+//            qreal distance = d->m_routingManager->routingModel()->totalDistance();
+//            unsigned int days = d->m_routingManager->routingModel()->duration().days;
+//            QTime time = d->m_routingManager->routingModel()->duration().time;
+//            QString timeString = time.toString( Qt::DefaultLocaleShortDate );
+//            if ( days ) {
+//                QString label = tr( "Estimated travel time: %1 days, %2 (%3 km)", 0, days );
+//                d->m_ui.descriptionLabel->setText( label.arg( days ).arg( timeString ).arg( distance, 0, 'f', 1 ) );
+//            } else {
+//                QString label = tr( "Estimated travel time: %1 (%2 km)" );
+//                d->m_ui.descriptionLabel->setText( label.arg( timeString ).arg( distance, 0, 'f', 1 ) );
+//            }
+//            d->m_ui.descriptionLabel->setVisible( true );
+
+            if ( d->m_zoomRouteAfterDownload ) {
+                d->m_zoomRouteAfterDownload = false;
+                d->m_widget->centerOn( GeoDataLatLonBox::fromLineString( bbox ) );
+            }
+        }
+    }
+
+    d->m_ui.routeComboBox->setVisible( d->m_ui.routeComboBox->count() > 0 );
+    if ( d->m_ui.routeComboBox->currentIndex() < 0 && d->m_ui.routeComboBox->count() > 0 ) {
+        d->m_ui.routeComboBox->setCurrentIndex( 0 );
+    }
+
+    d->m_progressTimer.stop();
+    d->m_ui.searchButton->setIcon( QIcon() );
+}
+
+void RoutingWidget::setOpenFileButtonVisible( bool visible )
+{
+    d->m_ui.openButton->setVisible( visible );
+}
+
+void RoutingWidget::openRouteFile()
+{
+    QString const file = QFileDialog::getOpenFileName( this, tr( "Open Route" ),
+                            QString(), tr("KML Files (*.kml)") );
+    if ( !file.isEmpty() ) {
+        d->m_routingManager->alternativeRoutesModel()->clear();
+        d->m_routingManager->loadRoute( file );
+    }
+}
+
+void RoutingWidget::selectFirstProfile()
+{
+    int count = d->m_routingManager->profilesModel()->rowCount();
+    if ( count && d->m_ui.routingProfileComboBox->currentIndex() < 0 ) {
+        d->m_ui.routingProfileComboBox->setCurrentIndex( 0 );
+    }
+}
+
+void RoutingWidget::setRoutingProfile( int index )
+{
+    if ( index >= 0 && index < d->m_routingManager->profilesModel()->rowCount() ) {
+        d->m_routeRequest->setRoutingProfile( d->m_routingManager->profilesModel()->profiles().at( index ) );
+    }
+}
+
 } // namespace Marble
 
 #include "RoutingWidget.moc"
