@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -60,6 +60,7 @@ inline QByteArray & QHttpHttpJob::data()
 class QHttpNetworkPlugin: public NetworkPlugin
 {
     Q_OBJECT
+    Q_INTERFACES( Marble::NetworkPluginInterface )
 
  public:
     virtual QString name() const;
