@@ -1,11 +1,11 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2008 Torsten Rahn <tackat@kde.org>"
+// Copyright 2008 Torsten Rahn <tackat@kde.org>
 //
 
 #ifndef MAPSCALEFLOATITEM_H
@@ -50,9 +50,10 @@ class MapScaleFloatItem : public AbstractFloatItem
 
     bool isInitialized () const;
 
-    bool needsUpdate( ViewportParams *viewport );
+    void changeViewport( ViewportParams *viewport );
 
-    bool renderFloatItem( GeoPainter *painter, ViewportParams *viewport, GeoSceneLayer * layer = 0 );
+    void paintContent( GeoPainter *painter, ViewportParams *viewport,
+                       const QString& renderPos, GeoSceneLayer * layer = 0 );
 
  private:
     int   invScale() const            { return m_invScale; }
