@@ -1,3 +1,4 @@
+
 /* This file is part of Rocs,
    Copyright (C) 2008 by:
    Tomaz Canabrava <tomaz.canabrava@gmail.com>
@@ -22,21 +23,21 @@
 #define ADDEDGE_H
 
 #include <QUndoCommand>
-class Edge;
-class Node;
-class Graph;
+class DataType;
+class Pointer;
+class Datum;
 
 
 class URAddEdge : public QUndoCommand {
 public:
-    URAddEdge(Edge *e);
+    URAddEdge(Pointer *e);
     void redo();
     void undo();
 private:
-    Edge *_edge;
-    Node *_from;
-    Node *_to;
-    Graph *_graph;
+    Pointer *_edge;
+    Datum *_from;
+    Datum *_to;
+    DataType *_graph;
 };
 
 #endif
